package net.alxeg.antask.rapc;

import java.io.*;
import java.util.*;

public class JDEProject extends JDEFile {
    private String[] dependences = null;
  private File[] files = null;

    public JDEProject(String fileName) throws FileNotFoundException, IOException {
        super(fileName);
        dependences = getPropertiesArray("DependsOn");
    getFiles();
    }

    public boolean notDepended() {
        return (dependences == null || dependences.length==0);
    }

    public String[] getDependences() {
        return dependences;
    }

  public void build() throws JDEBuildException {
    if (isBuildNeeded()) {
      log("Building "+getFileName()+"...");
      String commandLine = surroundQuotes(JDEBuildEnvironment.getInstance().getRapcExecutable())+" ";
      String options = getProperty("Options");
      if (options!=null) commandLine += options + " ";
      commandLine += "import="+surroundQuotes(JDEBuildEnvironment.getInstance().getRimLibrary());
      String imports = getImportsString();
      commandLine += imports;
      commandLine += " "+(getProperty("Type").equals("2")?"library=":"codename=")+getProperty("OutputFileName");
            try {
        generateRapcFile();
        commandLine += " "+getProperty("OutputFileName")+".rapc";
        for(int i=0; i<files.length; i++) {
          commandLine += " "+surroundQuotes(files[i].getAbsolutePath());
        }
        if (JDEBuildEnvironment.getInstance().getLogCmdLine()) {
          log("Executing: "+commandLine);
        }
        Process process = Runtime.getRuntime().exec(commandLine);
        new OutThread(System.out, process.getInputStream()).start();
        new OutThread(System.err, process.getErrorStream()).start();
        process.waitFor();
        int exitCode = process.exitValue();
        //log("Building "+getFileName()+" exited with code "+exitCode);
        if (exitCode==0) {
            log(getFileName()+" - no errors.");
        }
        else {
            throw new JDEBuildException("Build of "+getFileName()+" is failed");
        }

            }
            catch (JDEBuildException ex) {
                throw ex;
            }
            catch (Exception ex) {
        throw new JDEBuildException(ex.getMessage());
            }
    }
  }

  public String getImportsString() {
    String result = "";
    //if (result.length()>0 && result.charAt(result.length()-1)!=';') result += ";";

    if (dependences!=null)
      for(int i=0; i<dependences.length; i++) {
        try {
          JDEProject prj = new JDEProject(dependences[i] + ".jdp");
          String prjImports = prj.getImportsString();
          if (!prjImports.trim().equals("")) {
            result += prjImports.toLowerCase();
          }
        }
        catch (Exception ex) {
          ex.printStackTrace();
        }
        if (result.toLowerCase().indexOf((dependences[i]+".jar").toLowerCase())==-1 && !dependences[i].equals("")) {
          result += ";" + dependences[i]+".jar";
        }
      }

     String[] imports = getPropertiesArray("Imports");
     if (imports!=null)
     for(int i=0; i<imports.length; i++) {
        if (result.toLowerCase().indexOf(imports[i].toLowerCase())==-1 && !imports[i].equals(""))
            result += ";" + surroundQuotes(new File(imports[i]).getAbsolutePath());
     }
     return result;
  }

  private String surroundQuotes(String str) {
    if (str.indexOf(' ')!=-1) {
      str = "\""+str+"\"";
    }
    return str;
  }

  private void generateRapcFile() throws IOException {
    String projectName = getFileName();
    String outputName = getProperty("OutputFileName");
    File rapcFile = new File(outputName+".rapc");
    if (!rapcFile.exists() || new File(projectName).lastModified()>rapcFile.lastModified()) {
        FileOutputStream fs = new FileOutputStream(rapcFile, false);
        PrintWriter pw = new PrintWriter(fs);
        pw.println("MIDlet-Name: "+outputName);
        String version = getProperty("Version");
        pw.print("MIDlet-Version: ");
        if (version!=null) pw.println(version); else pw.println("0.0");
        String vendor = getProperty("Vendor");
        pw.print("MIDlet-Vendor: ");
        if (vendor!=null)   pw.println(vendor); else pw.println("<unknown>");
        pw.println("MIDlet-Jar-URL: "+outputName+".jar");
        pw.println("MIDlet-Jar-Size: 0");
        pw.println("MicroEdition-Profile: MIDP-1.0");
            pw.println("MicroEdition-Configuration: CLDC-1.0");
            String title = getProperty("Title");
            String midletClass = getProperty("MidletClass");
            String[] icons = getPropertiesArray("Icons");
            //if ( (icons!=null && icons.length>0) ||
            //       (midletClass!=null && !midletClass.equals("")) ||
            //       (title!=null && !title.equals("")) ) {

            pw.print("MIDlet-1: ");
            if (title!=null) pw.print(title);
            pw.print(",");
            if (icons!=null && icons.length>0) pw.print(icons[0]);
            pw.print(",");
            if (midletClass!=null) pw.print(midletClass);
            pw.println("");

            //}
            for (int i=1; i<icons.length; i++) {
                pw.println("RIM-MIDlet-Icon-1-"+i+": "+icons[i]);
            }
            if (icons.length>1) pw.println("RIM-MIDlet-Icon-Count-1: "+(icons.length-1));
            String pos = getProperty("RibbonPosition");
            if (pos!=null) pw.println("RIM-MIDlet-Position-1: "+pos);

            try {
            int flags = 0;
            String fSystem = getProperty("SystemModule");
            String fStartup = getProperty("RunOnStartup");
            String fTier = getProperty("StartupTier");
            if (fSystem!=null) {
                if (Integer.parseInt(fSystem)==1)
                    flags |=  0x2;
            }
            if (fStartup!=null) {
                if (Integer.parseInt(fStartup)==1)
                    flags |=  0x1;
            }
            if (fTier!=null) {
                switch (Integer.parseInt(fTier)) {
                    //TODO: add or change other startup tier options
                    case 7:
                      break;
                    case 6:
                        flags |=  0x20;
                        break;
                };
            }
            pw.println("RIM-MIDlet-Flags-1: "+flags);

        }
        catch(NumberFormatException ex) {}

      String titleResBundle = getProperty("TitleResourceBundle");
      String titleResId = getProperty("TitleResourceId");
      pw.print("RIM-MIDlet-NameResourceBundle-1: ");
      if (titleResBundle!=null) pw.println(titleResBundle); else pw.println("");
      pw.print("RIM-MIDlet-NameResourceId-1: ");
      if (titleResId!=null) pw.println(titleResId); else pw.println("0");

        pw.flush();
        fs.close();
    }

  }

  private boolean isBuildNeeded() {
    boolean result = false;
    String projectName = getFileName();
    File codFile = new File(projectName.substring(0, projectName.indexOf('.'))+".cod");
    if (codFile.exists()) {
      long codModified = codFile.lastModified();
      for(int i=0; i<files.length; i++) {
        if (files[i].lastModified()>codModified) {
          result = true;
          break;
        }
      }
      for(int i=0; i<dependences.length; i++) {
        if (new File(dependences[i]+".cod").lastModified()>codModified) {
          result = true;
          break;
        }
      }
    }
    else {
      result = true;
    }

    return result;
  }

  public File[] getFiles() {
    if (files==null) {
      Vector names = getPropertiesVector("Files");
      files = new File[names.size()]; int i=0;
      for (Iterator it = names.iterator(); it.hasNext(); ) {
        files[i++] = new File((String)it.next());
      }
    }
        return files;
  }

  public void log(String message) {
    System.out.println(message);
  }

  class OutThread extends Thread {
    InputStream is= null;
    PrintStream writer = null;
    public OutThread(PrintStream writer, InputStream is) {
      this.is = is;
      this.writer = writer;
    }
    public void run() {
      try {
        BufferedReader br = new BufferedReader(new InputStreamReader(is));
        String line=null;
        while ( (line = br.readLine()) != null)
          writer.println(line);
       }
       catch (IOException ex) {}
    }
  }


}
