package net.alxeg.antask.rapc;
import java.io.*;
import java.util.*;
public class JDEWorkspace extends JDEFile {
  
	public JDEWorkspace() {
	}
  
	public JDEWorkspace(String fileName)
		throws FileNotFoundException, IOException {
		super(fileName);
	}
  
	public String[] getBuildConfigurations() throws JDEBuildException {
		String[] configurations = getPropertiesArray("BuildConfigurations");
		if (configurations == null)
			throw new JDEBuildException("No build configurations found");
		return configurations;
	}
  
	Vector alreadyBuild;
  
	public void build(String config) throws JDEBuildException {
		try {
			String[] activeProjects = getPropertiesArray(config + "ActiveProjects");
			if (activeProjects == null || activeProjects.length == 0)
				throw new JDEBuildException(
					"No active projects for " + config + " configuration found");
			alreadyBuild = new Vector();
			for (int i = 0; i < activeProjects.length; i++) {
				JDEProject prj = new JDEProject(activeProjects[i]);
				build(prj);
			}
		}
		catch (JDEBuildException ex) {
			throw ex;
		}
		catch (Exception ex) {
			//ex.printStackTrace();
			throw new JDEBuildException(ex.toString());
		}
	}
    
    public void build(File[] projects) throws FileNotFoundException, JDEBuildException, IOException {
			alreadyBuild = new Vector();
      for (int i=0; i<projects.length; i++) {
        build(new JDEProject(projects[i].getAbsolutePath())); 
      }
    }
        
	private void build(JDEProject project) throws JDEBuildException {
		if (alreadyBuild.contains(project.getFileName().toLowerCase()))
			return;
		try {
			if (project.notDepended()) {
				project.build();
				alreadyBuild.add(project.getFileName().toLowerCase());
			}
			else {
				String[] dependences = project.getDependences();
				for (int i = 0; i < dependences.length; i++) {
					JDEProject prj = new JDEProject(dependences[i] + ".jdp");
					build(prj);
				}
				project.build();
				alreadyBuild.add(project.getFileName().toLowerCase());
			}
		}
		catch (JDEBuildException ex) {
			throw ex;
		}
		catch (Exception ex) {
			ex.printStackTrace();
			throw new JDEBuildException(ex.toString());
		}
	}
  
	public static void main(String[] args) throws Throwable {
		if (args.length < 1)
			return;
		JDEBuildEnvironment.getInstance().setJdeHome(
			"D:\\Program Files\\Research In Motion\\BlackBerry JDE 3.7");
		new JDEWorkspace(args[0]).build("Release");
	}
}