package net.alxeg.antask.rapc;

import org.apache.tools.ant.types.*;
import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.DirectoryScanner;
import org.apache.tools.ant.Task;
import java.io.*;
import java.util.*;

/**
 *
 */
public class RapcTask extends Task {
    private String workspace = null;
    private String jdePath = null;
    private String buildConfiguration = null;
    private String logCommandLine = "false";

    protected Vector filesets = new Vector();

    private void verifyParameters() throws BuildException {
      if ( (filesets.size()!=0 && workspace!=null) ||
           (filesets.size()==0 && workspace==null) ) {
        throw new BuildException("Must be specified either workspace name or fileset of projects");
      }
      if (workspace!=null && buildConfiguration==null) {
        throw new BuildException("No build configuration specified");
      }
      if (jdePath==null) {
        throw new BuildException("No jdePath specified");
      }
    }

    public void execute() throws BuildException {
      verifyParameters();
      JDEBuildEnvironment.getInstance().setJdeHome(jdePath);
      if (logCommandLine.equalsIgnoreCase("true"))
          JDEBuildEnvironment.getInstance().setLogCmdLine(true);
        else
          JDEBuildEnvironment.getInstance().setLogCmdLine(false);

      try {
        if (filesets.size()==0) {
          JDEWorkspace ws = new JDEWorkspace(workspace);
          ws.build(buildConfiguration);
        }
        else {
          for (int i = 0; i < filesets.size(); i++) {
            FileSet fs = (FileSet) filesets.elementAt(i);
            try {
              DirectoryScanner ds = fs.getDirectoryScanner(getProject());
              String[] files = ds.getIncludedFiles();
              buildProjects(fs.getDir(getProject()), files);
            }
            catch (BuildException be) {
              throw be;
            }
          }
        }
      }
      catch(Exception ex) {
        ex.printStackTrace();
        throw new BuildException(ex.getMessage());
      }
    }

    protected void buildProjects(File d, String[] fileNames) throws FileNotFoundException, JDEBuildException, IOException {
      File[] files = new File[fileNames.length];
      for (int j = 0; j < fileNames.length; j++) {
        File f = new File(d, fileNames[j]);
        //log("====== "+f.getAbsolutePath());
        files[j] = f;
      }
      System.getProperties().put("user.dir", d.getAbsolutePath());
      JDEWorkspace ws= new JDEWorkspace();
      ws.build(files);
    }

    public void addFileset(FileSet set) {
      filesets.addElement(set);
    }


    public void setWorkspace(String workspace) {
      this.workspace = workspace;
    }

    public void setJdePath(String jdePath) {
      this.jdePath = jdePath;
        }
        public void setLogCommandLine(String logCommandLine) {
            this.logCommandLine = logCommandLine;
        }

    public void setBuildConfiguration(String buildConfiguration) {
      this.buildConfiguration = buildConfiguration;
    }

}